@@ -17,8 +17,8 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-#ifndef _MLTPP_REPOSITORY_H_
-#define _MLTPP_REPOSITORY_H_
+#ifndef MLTPP_REPOSITORY_H
+#define MLTPP_REPOSITORY_H
 
 #include "config.h"
 
