@@ -1,6 +1,6 @@
 /*
  * filter_ladspa.c -- filter audio through LADSPA plugins
- * Copyright (C) 2004-2014 Meltytech, LLC
+ * Copyright (C) 2004-2016 Meltytech, LLC
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -61,9 +61,20 @@ static jack_rack_t* initialise_jack_rack( mlt_properties properties, int channel
 			{
 				plugin->enabled = TRUE;
 				process_add_plugin( jackrack->procinfo, plugin );
+				mlt_properties_set_int( properties, "instances", plugin->copies );
 			}
 			else
+			{
 				mlt_log_error( properties, "failed to load plugin %lu\n", id );
+				return jackrack;
+			}
+
+			if ( plugin && plugin->desc && plugin->copies == 0 )
+			{
+				mlt_log_warning( properties, "Not compatible with %d channels. Requesting %lu channels instead.\n",
+					channels, plugin->desc->channels );
+				jackrack = initialise_jack_rack( properties, plugin->desc->channels );
+			}
 		}
 	}
 	return jackrack;
@@ -75,16 +86,14 @@ static jack_rack_t* initialise_jack_rack( mlt_properties properties, int channel
 
 static int ladspa_get_audio( mlt_frame frame, void **buffer, mlt_audio_format *format, int *frequency, int *channels, int *samples )
 {
+	int error = 0;
+
 	// Get the filter service
 	mlt_filter filter = mlt_frame_pop_audio( frame );
 
 	// Get the filter properties
 	mlt_properties filter_properties = MLT_FILTER_PROPERTIES( filter );
 
-	// Get the producer's audio
-	*format = mlt_audio_float;
-	mlt_frame_get_audio( frame, buffer, format, frequency, channels, samples );
-	
 	// Initialise LADSPA if needed
 	jack_rack_t *jackrack = mlt_properties_get_data( filter_properties, "jackrack", NULL );
 	if ( jackrack == NULL )
@@ -98,20 +107,25 @@ static int ladspa_get_audio( mlt_frame frame, void **buffer, mlt_audio_format *f
 	{
 		plugin_t *plugin = jackrack->procinfo->chain;
 		LADSPA_Data value;
-		int index, c;
+		int i, c;
 		mlt_position position = mlt_filter_get_position( filter, frame );
 		mlt_position length = mlt_filter_get_length2( filter, frame );
 
-		for ( index = 0; index < plugin->desc->control_port_count; index++ )
+		// Get the producer's audio
+		*channels = jackrack->channels;
+		*format = mlt_audio_float;
+		mlt_frame_get_audio( frame, buffer, format, frequency, channels, samples );
+
+		for ( i = 0; i < plugin->desc->control_port_count; i++ )
 		{
 			// Apply the control port values
 			char key[20];
-			value = plugin_desc_get_default_control_value( plugin->desc, index, sample_rate );
-			snprintf( key, sizeof(key), "%d", index );
+			value = plugin_desc_get_default_control_value( plugin->desc, i, sample_rate );
+			snprintf( key, sizeof(key), "%d", i );
 			if ( mlt_properties_get( filter_properties, key ) )
 				value = mlt_properties_anim_get_double( filter_properties, key, position, length );
 			for ( c = 0; c < plugin->copies; c++ )
-				plugin->holders[c].control_memory[index] = value;
+				plugin->holders[c].control_memory[i] = value;
 		}
 		plugin->wet_dry_enabled = mlt_properties_get( filter_properties, "wetness" ) != NULL;
 		if ( plugin->wet_dry_enabled )
@@ -120,25 +134,42 @@ static int ladspa_get_audio( mlt_frame frame, void **buffer, mlt_audio_format *f
 			for ( c = 0; c < *channels; c++ )
 				plugin->wet_dry_values[c] = value;
 		}
+
+		// Configure the buffers
+		LADSPA_Data **input_buffers  = mlt_pool_alloc( sizeof( LADSPA_Data* ) * *channels );
+		LADSPA_Data **output_buffers = mlt_pool_alloc( sizeof( LADSPA_Data* ) * *channels );
+
+		for ( i = 0; i < *channels; i++ )
+		{
+			input_buffers[i]  = (LADSPA_Data*) *buffer + i * *samples;
+			output_buffers[i] = (LADSPA_Data*) *buffer + i * *samples;
+		}
+
+		// Do LADSPA processing
+		error = process_ladspa( jackrack->procinfo, *samples, input_buffers, output_buffers );
+
+		mlt_pool_release( input_buffers );
+		mlt_pool_release( output_buffers );
+
+		// read the status port values
+		for ( i = 0; i < plugin->desc->status_port_count; i++ )
+		{
+			char key[20];
+			int p = plugin->desc->status_port_indicies[i];
+			for ( c = 0; c < plugin->copies; c++ )
+			{
+				snprintf( key, sizeof(key), "%d[%d]", p, c );
+				value = plugin->holders[c].status_memory[i];
+				mlt_properties_set_double( filter_properties, key, value );
+			}
+		}
 	}
-
-	// Get the filter-specific properties
-	LADSPA_Data **input_buffers  = mlt_pool_alloc( sizeof( LADSPA_Data* ) * *channels );
-	LADSPA_Data **output_buffers = mlt_pool_alloc( sizeof( LADSPA_Data* ) * *channels );
-
-	int i;
-	for ( i = 0; i < *channels; i++ )
+	else
 	{
-		input_buffers[i]  = (LADSPA_Data*) *buffer + i * *samples;
-		output_buffers[i] = (LADSPA_Data*) *buffer + i * *samples;
+		// Nothing to do.
+		error = mlt_frame_get_audio( frame, buffer, format, frequency, channels, samples );
 	}
 
-	// Do LADSPA processing
-	int error = jackrack && process_ladspa( jackrack->procinfo, *samples, input_buffers, output_buffers );
-
-	mlt_pool_release( input_buffers );
-	mlt_pool_release( output_buffers );
-
 	return error;
 }
 
