@@ -9,7 +9,7 @@
 #include <stdlib.h>
 
 /* This can be replaced by any BSD-like queue implementation. */
-#include "queue.h"
+#include <sys/queue.h>
 
 #ifdef USE_SPEEX_RESAMPLER
   #include <speex/speex_resampler.h>
@@ -21,10 +21,10 @@
     goto goto_point;                                                           \
   }
 
-SLIST_HEAD(ebur128_double_queue, ebur128_dq_entry);
+STAILQ_HEAD(ebur128_double_queue, ebur128_dq_entry);
 struct ebur128_dq_entry {
   double z;
-  SLIST_ENTRY(ebur128_dq_entry) entries;
+  STAILQ_ENTRY(ebur128_dq_entry) entries;
 };
 
 struct ebur128_state_internal {
@@ -34,6 +34,8 @@ struct ebur128_state_internal {
   size_t audio_data_frames;
   /** Current index for audio_data. */
   size_t audio_data_index;
+  /** Number of audio frames in audio_data. */
+  size_t audio_data_frame_count;
   /** How many frames are needed for a gating block. Will correspond to 400ms
    *  of audio at initialization, and 100ms after the first block (75% overlap
    *  as specified in the 2011 revision of BS1770). */
@@ -50,8 +52,12 @@ struct ebur128_state_internal {
   double v[5][5];
   /** Linked list of block energies. */
   struct ebur128_double_queue block_list;
+  unsigned int block_list_max;
+  unsigned int block_list_size;
   /** Linked list of 3s-block energies, used to calculate LRA. */
   struct ebur128_double_queue short_term_block_list;
+  unsigned int st_block_list_max;
+  unsigned int st_block_list_size;
   int use_histogram;
   unsigned long *block_energy_histogram;
   unsigned long *short_term_block_energy_histogram;
@@ -232,7 +238,10 @@ void ebur128_get_version(int* major, int* minor, int* patch) {
 ebur128_state* ebur128_init(unsigned int channels,
                             unsigned long samplerate,
                             int mode) {
-  int errcode, result;
+#ifdef USE_SPEEX_RESAMPLER
+  int result;
+#endif
+  int errcode;
   ebur128_state* st;
   unsigned int i;
 
@@ -255,21 +264,29 @@ ebur128_state* ebur128_init(unsigned int channels,
   }
 
   st->d->use_histogram = mode & EBUR128_MODE_HISTOGRAM ? 1 : 0;
-
+  st->history = UINT_MAX;
   st->samplerate = samplerate;
   st->d->samples_in_100ms = (st->samplerate + 5) / 10;
   st->mode = mode;
   if ((mode & EBUR128_MODE_S) == EBUR128_MODE_S) {
-    st->d->audio_data_frames = st->d->samples_in_100ms * 30;
+    st->window = 3000;
   } else if ((mode & EBUR128_MODE_M) == EBUR128_MODE_M) {
-    st->d->audio_data_frames = st->d->samples_in_100ms * 4;
+    st->window = 400;
   } else {
     goto free_true_peak;
   }
+  st->d->audio_data_frames = st->samplerate * st->window / 1000;
+  if (st->d->audio_data_frames % st->d->samples_in_100ms) {
+    /* round up to multiple of samples_in_100ms */
+    st->d->audio_data_frames = st->d->audio_data_frames
+                             + st->d->samples_in_100ms
+                             - (st->d->audio_data_frames % st->d->samples_in_100ms);
+  }
   st->d->audio_data = (double*) malloc(st->d->audio_data_frames *
                                        st->channels *
                                        sizeof(double));
   CHECK_ERROR(!st->d->audio_data, 0, free_true_peak)
+  st->d->audio_data_frame_count = 0;
   ebur128_init_filter(st);
 
   if (st->d->use_histogram) {
@@ -290,8 +307,12 @@ ebur128_state* ebur128_init(unsigned int channels,
   } else {
     st->d->short_term_block_energy_histogram = NULL;
   }
-  SLIST_INIT(&st->d->block_list);
-  SLIST_INIT(&st->d->short_term_block_list);
+  STAILQ_INIT(&st->d->block_list);
+  st->d->block_list_size = 0;
+  st->d->block_list_max = st->history / 100;
+  STAILQ_INIT(&st->d->short_term_block_list);
+  st->d->st_block_list_size = 0;
+  st->d->st_block_list_max = st->history / 3000;
   st->d->short_term_frame_counter = 0;
 
 #ifdef USE_SPEEX_RESAMPLER
@@ -319,8 +340,10 @@ ebur128_state* ebur128_init(unsigned int channels,
 
   return st;
 
+#ifdef USE_SPEEX_RESAMPLER
 free_short_term_block_energy_histogram:
   free(st->d->short_term_block_energy_histogram);
+#endif
 free_block_energy_histogram:
   free(st->d->block_energy_histogram);
 free_audio_data:
@@ -347,14 +370,14 @@ void ebur128_destroy(ebur128_state** st) {
   free((*st)->d->channel_map);
   free((*st)->d->sample_peak);
   free((*st)->d->true_peak);
-  while (!SLIST_EMPTY(&(*st)->d->block_list)) {
-    entry = SLIST_FIRST(&(*st)->d->block_list);
-    SLIST_REMOVE_HEAD(&(*st)->d->block_list, entries);
+  while (!STAILQ_EMPTY(&(*st)->d->block_list)) {
+    entry = STAILQ_FIRST(&(*st)->d->block_list);
+    STAILQ_REMOVE_HEAD(&(*st)->d->block_list, entries);
     free(entry);
   }
-  while (!SLIST_EMPTY(&(*st)->d->short_term_block_list)) {
-    entry = SLIST_FIRST(&(*st)->d->short_term_block_list);
-    SLIST_REMOVE_HEAD(&(*st)->d->short_term_block_list, entries);
+  while (!STAILQ_EMPTY(&(*st)->d->short_term_block_list)) {
+    entry = STAILQ_FIRST(&(*st)->d->short_term_block_list);
+    STAILQ_REMOVE_HEAD(&(*st)->d->short_term_block_list, entries);
     free(entry);
   }
 #ifdef USE_SPEEX_RESAMPLER
@@ -456,7 +479,7 @@ static void ebur128_filter_##type(ebur128_state* st, const type* src,          \
   for (c = 0; c < st->channels; ++c) {                                         \
     int ci = st->d->channel_map[c] - 1;                                        \
     if (ci < 0) continue;                                                      \
-    else if (ci > 4) ci = 0; /* dual mono */                                   \
+    else if (ci == EBUR128_DUAL_MONO - 1) ci = 0; /*dual mono */               \
     for (i = 0; i < frames; ++i) {                                             \
       st->d->v[ci][0] = (double) (src[i * st->channels + c] / scaling_factor)  \
                    - st->d->a[1] * st->d->v[ci][1]                             \
@@ -532,8 +555,12 @@ static int ebur128_calc_gating_block(ebur128_state* st, size_t frames_per_block,
                        st->d->audio_data[i * st->channels + c];
       }
     }
-    if (st->d->channel_map[c] == EBUR128_LEFT_SURROUND ||
-        st->d->channel_map[c] == EBUR128_RIGHT_SURROUND) {
+    if (st->d->channel_map[c] == EBUR128_Mp110 ||
+        st->d->channel_map[c] == EBUR128_Mm110 ||
+        st->d->channel_map[c] == EBUR128_Mp060 ||
+        st->d->channel_map[c] == EBUR128_Mm060 ||
+        st->d->channel_map[c] == EBUR128_Mp090 ||
+        st->d->channel_map[c] == EBUR128_Mm090) {
       channel_sum *= 1.41;
     } else if (st->d->channel_map[c] == EBUR128_DUAL_MONO) {
       channel_sum *= 2.0;
@@ -549,10 +576,16 @@ static int ebur128_calc_gating_block(ebur128_state* st, size_t frames_per_block,
       ++st->d->block_energy_histogram[find_histogram_index(sum)];
     } else {
       struct ebur128_dq_entry* block;
-      block = (struct ebur128_dq_entry*) malloc(sizeof(struct ebur128_dq_entry));
-      if (!block) return EBUR128_ERROR_NOMEM;
+      if (st->d->block_list_size == st->d->block_list_max) {
+        block = STAILQ_FIRST(&st->d->block_list);
+        STAILQ_REMOVE_HEAD(&st->d->block_list, entries);
+      } else {
+        block = (struct ebur128_dq_entry*) malloc(sizeof(struct ebur128_dq_entry));
+        if (!block) return EBUR128_ERROR_NOMEM;
+        st->d->block_list_size++;
+      }
       block->z = sum;
-      SLIST_INSERT_HEAD(&st->d->block_list, block, entries);
+      STAILQ_INSERT_TAIL(&st->d->block_list, block, entries);
     }
     return EBUR128_SUCCESS;
   } else {
@@ -578,10 +611,10 @@ int ebur128_set_channel(ebur128_state* st,
 int ebur128_change_parameters(ebur128_state* st,
                               unsigned int channels,
                               unsigned long samplerate) {
-  int errcode;
+  int errcode = EBUR128_SUCCESS;
   if (channels == st->channels &&
       samplerate == st->samplerate) {
-    return 2;
+    return EBUR128_ERROR_NO_CHANGE;
   }
   free(st->d->audio_data);
   st->d->audio_data = NULL;
@@ -594,11 +627,6 @@ int ebur128_change_parameters(ebur128_state* st,
     free(st->d->true_peak);   st->d->true_peak = NULL;
     st->channels = channels;
 
-#ifdef USE_SPEEX_RESAMPLER
-    ebur128_destroy_resampler(st);
-    ebur128_init_resampler(st);
-#endif
-
     errcode = ebur128_init_channel_map(st);
     CHECK_ERROR(errcode, EBUR128_ERROR_NOMEM, exit)
 
@@ -613,20 +641,27 @@ int ebur128_change_parameters(ebur128_state* st,
   }
   if (samplerate != st->samplerate) {
     st->samplerate = samplerate;
+    st->d->samples_in_100ms = (st->samplerate + 5) / 10;
     ebur128_init_filter(st);
   }
-  if ((st->mode & EBUR128_MODE_S) == EBUR128_MODE_S) {
-    st->d->audio_data_frames = st->d->samples_in_100ms * 30;
-  } else if ((st->mode & EBUR128_MODE_M) == EBUR128_MODE_M) {
-    st->d->audio_data_frames = st->d->samples_in_100ms * 4;
-  } else {
-    return 1;
+  st->d->audio_data_frames = st->samplerate * st->window / 1000;
+  if (st->d->audio_data_frames % st->d->samples_in_100ms) {
+    /* round up to multiple of samples_in_100ms */
+    st->d->audio_data_frames = st->d->audio_data_frames
+                             + st->d->samples_in_100ms
+                             - (st->d->audio_data_frames % st->d->samples_in_100ms);
   }
   st->d->audio_data = (double*) malloc(st->d->audio_data_frames *
                                        st->channels *
                                        sizeof(double));
   CHECK_ERROR(!st->d->audio_data, EBUR128_ERROR_NOMEM, exit)
 
+#ifdef USE_SPEEX_RESAMPLER
+  ebur128_destroy_resampler(st);
+  errcode = ebur128_init_resampler(st);
+  CHECK_ERROR(errcode, EBUR128_ERROR_NOMEM, exit)
+#endif
+
   /* the first block needs 400ms of audio data */
   st->d->needed_frames = st->d->samples_in_100ms * 4;
   /* start at the beginning of the buffer */
@@ -634,12 +669,77 @@ int ebur128_change_parameters(ebur128_state* st,
   /* reset short term frame counter */
   st->d->short_term_frame_counter = 0;
 
-  return 0;
-
 exit:
-  return 1;
+  return errcode;
 }
 
+int ebur128_set_max_window(ebur128_state* st, unsigned int window)
+{
+  int errcode = EBUR128_SUCCESS;
+
+  if ((st->mode & EBUR128_MODE_S) == EBUR128_MODE_S && window < 3000) {
+    window = 3000;
+  } else if ((st->mode & EBUR128_MODE_M) == EBUR128_MODE_M && window < 400) {
+    window = 400;
+  }
+  if (window == st->window) {
+    return EBUR128_ERROR_NO_CHANGE;
+  }
+
+  st->window = window;
+  free(st->d->audio_data);
+  st->d->audio_data = NULL;
+  st->d->audio_data_frames = st->samplerate * st->window / 1000;
+  if (st->d->audio_data_frames % st->d->samples_in_100ms) {
+    /* round up to multiple of samples_in_100ms */
+    st->d->audio_data_frames = st->d->audio_data_frames
+                             + st->d->samples_in_100ms
+                             - (st->d->audio_data_frames % st->d->samples_in_100ms);
+  }
+  st->d->audio_data = (double*) malloc(st->d->audio_data_frames *
+                                       st->channels *
+                                       sizeof(double));
+  CHECK_ERROR(!st->d->audio_data, EBUR128_ERROR_NOMEM, exit)
+  st->d->audio_data_frame_count = 0;
+
+  /* the first block needs 400ms of audio data */
+  st->d->needed_frames = st->d->samples_in_100ms * 4;
+  /* start at the beginning of the buffer */
+  st->d->audio_data_index = 0;
+  /* reset short term frame counter */
+  st->d->short_term_frame_counter = 0;
+
+exit:
+  return errcode;
+}
+
+int ebur128_set_max_history(ebur128_state* st, unsigned int history)
+{
+  if ((st->mode & EBUR128_MODE_LRA) == EBUR128_MODE_LRA && history < 3000) {
+    history = 3000;
+  } else if ((st->mode & EBUR128_MODE_M) == EBUR128_MODE_M && history < 400) {
+    history = 400;
+  }
+  if (history == st->history) {
+    return EBUR128_ERROR_NO_CHANGE;
+  }
+  st->history = history;
+  st->d->block_list_max = st->history / 100;
+  st->d->st_block_list_max = st->history / 3000;
+  while (st->d->block_list_size > st->d->block_list_max) {
+    struct ebur128_dq_entry* block = STAILQ_FIRST(&st->d->block_list);
+    STAILQ_REMOVE_HEAD(&st->d->block_list, entries);
+    free(block);
+    st->d->block_list_size--;
+  }
+  while (st->d->st_block_list_size > st->d->st_block_list_max) {
+    struct ebur128_dq_entry* block = STAILQ_FIRST(&st->d->short_term_block_list);
+    STAILQ_REMOVE_HEAD(&st->d->short_term_block_list, entries);
+    free(block);
+    st->d->st_block_list_size--;
+  }
+  return EBUR128_SUCCESS;
+}
 
 static int ebur128_energy_shortterm(ebur128_state* st, double* out);
 #define EBUR128_ADD_FRAMES(type)                                               \
@@ -652,6 +752,9 @@ int ebur128_add_frames_##type(ebur128_state* st,                               \
       src_index += st->d->needed_frames * st->channels;                        \
       frames -= st->d->needed_frames;                                          \
       st->d->audio_data_index += st->d->needed_frames * st->channels;          \
+      if (st->d->audio_data_frame_count < st->d->audio_data_frames ) {         \
+        st->d->audio_data_frame_count += st->d->needed_frames;                 \
+      }                                                                        \
       /* calculate the new gating block */                                     \
       if ((st->mode & EBUR128_MODE_I) == EBUR128_MODE_I) {                     \
         if (ebur128_calc_gating_block(st, st->d->samples_in_100ms * 4, NULL)) {\
@@ -669,11 +772,18 @@ int ebur128_add_frames_##type(ebur128_state* st,                               \
               ++st->d->short_term_block_energy_histogram[                      \
                                               find_histogram_index(st_energy)];\
             } else {                                                           \
-              block = (struct ebur128_dq_entry*)                               \
-                      malloc(sizeof(struct ebur128_dq_entry));                 \
-              if (!block) return EBUR128_ERROR_NOMEM;                          \
+              if (st->d->st_block_list_size == st->d->st_block_list_max) {     \
+                block = STAILQ_FIRST(&st->d->short_term_block_list);           \
+                STAILQ_REMOVE_HEAD(&st->d->short_term_block_list, entries);    \
+              } else {                                                         \
+                block = (struct ebur128_dq_entry*)                             \
+                        malloc(sizeof(struct ebur128_dq_entry));               \
+                if (!block) return EBUR128_ERROR_NOMEM;                        \
+                st->d->st_block_list_size++;                                   \
+              }                                                                \
               block->z = st_energy;                                            \
-              SLIST_INSERT_HEAD(&st->d->short_term_block_list, block, entries);\
+              STAILQ_INSERT_TAIL(&st->d->short_term_block_list,                \
+                                 block, entries);                              \
             }                                                                  \
           }                                                                    \
           st->d->short_term_frame_counter = st->d->samples_in_100ms * 20;      \
@@ -688,6 +798,9 @@ int ebur128_add_frames_##type(ebur128_state* st,                               \
     } else {                                                                   \
       ebur128_filter_##type(st, src + src_index, frames);                      \
       st->d->audio_data_index += frames * st->channels;                        \
+      if (st->d->audio_data_frame_count < st->d->audio_data_frames ) {         \
+        st->d->audio_data_frame_count += frames;                               \
+      }                                                                        \
       if ((st->mode & EBUR128_MODE_LRA) == EBUR128_MODE_LRA) {                 \
         st->d->short_term_frame_counter += frames;                             \
       }                                                                        \
@@ -702,12 +815,41 @@ EBUR128_ADD_FRAMES(int)
 EBUR128_ADD_FRAMES(float)
 EBUR128_ADD_FRAMES(double)
 
+static int ebur128_calc_relative_threshold(ebur128_state* st,
+                                           size_t* above_thresh_counter,
+                                           double* relative_threshold) {
+  struct ebur128_dq_entry* it;
+  size_t i;
+  *relative_threshold = 0.0;
+  *above_thresh_counter = 0;
+
+  if (st->d->use_histogram) {
+    for (i = 0; i < 1000; ++i) {
+      *relative_threshold += st->d->block_energy_histogram[i] *
+                            histogram_energies[i];
+      *above_thresh_counter += st->d->block_energy_histogram[i];
+    }
+  } else {
+    STAILQ_FOREACH(it, &st->d->block_list, entries) {
+      ++*above_thresh_counter;
+      *relative_threshold += it->z;
+    }
+  }
+
+  if (*above_thresh_counter != 0) {
+    *relative_threshold /= (double) *above_thresh_counter;
+    *relative_threshold *= relative_gate_factor;
+  }
+
+  return EBUR128_SUCCESS;
+}
+
 static int ebur128_gated_loudness(ebur128_state** sts, size_t size,
                                   double* out) {
   struct ebur128_dq_entry* it;
-  double relative_threshold = 0.0;
   double gated_loudness = 0.0;
-  size_t above_thresh_counter = 0;
+  double relative_threshold;
+  size_t above_thresh_counter;
   size_t i, j, start_index;
 
   for (i = 0; i < size; i++) {
@@ -718,25 +860,13 @@ static int ebur128_gated_loudness(ebur128_state** sts, size_t size,
 
   for (i = 0; i < size; i++) {
     if (!sts[i]) continue;
-    if (sts[i]->d->use_histogram) {
-      for (j = 0; j < 1000; ++j) {
-        relative_threshold += sts[i]->d->block_energy_histogram[j] *
-                              histogram_energies[j];
-        above_thresh_counter += sts[i]->d->block_energy_histogram[j];
-      }
-    } else {
-      SLIST_FOREACH(it, &sts[i]->d->block_list, entries) {
-        ++above_thresh_counter;
-        relative_threshold += it->z;
-      }
-    }
+    ebur128_calc_relative_threshold(sts[i], &above_thresh_counter, &relative_threshold);
   }
   if (!above_thresh_counter) {
     *out = -HUGE_VAL;
     return EBUR128_SUCCESS;
   }
-  relative_threshold /= (double) above_thresh_counter;
-  relative_threshold *= relative_gate_factor;
+
   above_thresh_counter = 0;
   if (relative_threshold < histogram_energy_boundaries[0]) {
     start_index = 0;
@@ -755,7 +885,7 @@ static int ebur128_gated_loudness(ebur128_state** sts, size_t size,
         above_thresh_counter += sts[i]->d->block_energy_histogram[j];
       }
     } else {
-      SLIST_FOREACH(it, &sts[i]->d->block_list, entries) {
+      STAILQ_FOREACH(it, &sts[i]->d->block_list, entries) {
         if (it->z >= relative_threshold) {
           ++above_thresh_counter;
           gated_loudness += it->z;
@@ -772,6 +902,24 @@ static int ebur128_gated_loudness(ebur128_state** sts, size_t size,
   return EBUR128_SUCCESS;
 }
 
+int ebur128_relative_threshold(ebur128_state* st, double* out) {
+  double relative_threshold;
+  size_t above_thresh_counter;
+
+  if (st && (st->mode & EBUR128_MODE_I) != EBUR128_MODE_I)
+    return EBUR128_ERROR_INVALID_MODE;
+
+  ebur128_calc_relative_threshold(st, &above_thresh_counter, &relative_threshold);
+
+  if (!above_thresh_counter) {
+      *out = -70.0;
+      return EBUR128_SUCCESS;
+  }
+
+  *out = ebur128_energy_to_loudness(relative_threshold);
+  return EBUR128_SUCCESS;
+}
+
 int ebur128_loudness_global(ebur128_state* st, double* out) {
   return ebur128_gated_loudness(&st, 1, out);
 }
@@ -787,6 +935,9 @@ static int ebur128_energy_in_interval(ebur128_state* st,
   if (interval_frames > st->d->audio_data_frames) {
     return EBUR128_ERROR_INVALID_MODE;
   }
+  if (interval_frames > st->d->audio_data_frame_count) {
+    return EBUR128_ERROR_INSUFFICIENT_DATA;
+  }
   ebur128_calc_gating_block(st, interval_frames, out);
   return EBUR128_SUCCESS;
 }
@@ -822,6 +973,22 @@ int ebur128_loudness_shortterm(ebur128_state* st, double* out) {
   return EBUR128_SUCCESS;
 }
 
+int ebur128_loudness_window(ebur128_state* st,
+                            unsigned int window,
+                            double* out) {
+  double energy;
+  size_t interval_frames = st->samplerate * window / 1000;
+  int error = ebur128_energy_in_interval(st, interval_frames, &energy);
+  if (error) {
+    return error;
+  } else if (energy <= 0.0) {
+    *out = -HUGE_VAL;
+    return EBUR128_SUCCESS;
+  }
+  *out = ebur128_energy_to_loudness(energy);
+  return EBUR128_SUCCESS;
+}
+
 static int ebur128_double_cmp(const void *p1, const void *p2) {
   const double* d1 = (const double*) p1;
   const double* d2 = (const double*) p2;
@@ -916,7 +1083,7 @@ int ebur128_loudness_range_multiple(ebur128_state** sts, size_t size,
     stl_size = 0;
     for (i = 0; i < size; ++i) {
       if (!sts[i]) continue;
-      SLIST_FOREACH(it, &sts[i]->d->short_term_block_list, entries) {
+      STAILQ_FOREACH(it, &sts[i]->d->short_term_block_list, entries) {
         ++stl_size;
       }
     }
@@ -930,7 +1097,7 @@ int ebur128_loudness_range_multiple(ebur128_state** sts, size_t size,
 
     for (j = 0, i = 0; i < size; ++i) {
       if (!sts[i]) continue;
-      SLIST_FOREACH(it, &sts[i]->d->short_term_block_list, entries) {
+      STAILQ_FOREACH(it, &sts[i]->d->short_term_block_list, entries) {
         stl_vector[j] = it->z;
         ++j;
       }
